<?php
/**
 * Pluggable functions for status
 * @since 1.6.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Are we using the status setting?
 * @since 1.6.0
 * @return Boolean
 */
if( ! function_exists( 'ctdb_is_status_enabled' ) ) {
	function ctdb_is_status_enabled() {
		// Status is enabled by selecting it in the metafields
		$metafields = ctdb_selected_meta_fields();
		if( isset( $metafields['status'] ) ) {
			return true;
		}
		return false;
	}
}
	
/**
 * Check if statuses are enabled
 * @since 1.6.0
 * @return String
 */
if( ! function_exists( 'ctdb_status_scope' ) ) {
	function ctdb_status_scope() {
		$pro_options = get_option( 'ctdb_pro_settings');
		if( isset( $pro_options['status_scope'] ) ) {
			// Return the status setting if enabled
			return $pro_options['status_scope'];
		}
		// This is the default value
		return 'global';
	}
}

/**
 * Get status of current topic
 * @since 1.6.0
 * @return String
 */
if( ! function_exists( 'ctdb_status' ) ) {
	function ctdb_status( $post_id=false ) {
		if( false === $post_id ) {
			$post_id = get_the_ID();
		}
		// Check for status metafield
		$status = get_post_meta( $post_id, 'ctdb_topic_status', true );
		// If status has not been set yet, set the default
		if( empty( $status ) ) {
			$status = ctdb_default_status();
			update_post_meta( $post_id, 'ctdb_topic_status', sanitize_text_field( $status ) );
		}
		return $status;
	}
}

/**
 * Get list of statuses
 * @since 1.6.0
 * @return Array
 */
if( ! function_exists( 'ctdb_statuses_list' ) ) {
	function ctdb_statuses_list() {
		$statuses = array();
		// Check the scope
		if( ctdb_status_scope() == 'board' ) {
			// Board statuses - different statuses per board
			$post_id = get_the_ID();
			$board_id = ctdb_topic_board_id( $post_id );
			// Check if board has statuses
			$statuses = get_term_meta( $board_id, 'ctdb_board_statuses', true );
			if( ! empty( $statuses ) ) {
				$statuses = explode( ',', $statuses );
			} else {
				// Otherwise use default global statuses
				$statuses = ctdb_global_statuses();
			}
		} else {
			// Global statuses - same for all boards and topics
			$statuses = ctdb_global_statuses();
		}
		
		return $statuses;
	}
}

/**
 * Get the global statuses
 * @since 1.6.0
 * @return Array
 */
if( ! function_exists( 'ctdb_global_statuses' ) ) {
	function ctdb_global_statuses() {
		$statuses = array();
		$pro_options = get_option( 'ctdb_pro_settings');
		if( isset( $pro_options['default_statuses'] ) ) {
			$statuses = explode( ',', $pro_options['default_statuses'] );
		}
		return $statuses;
	}
}

/**
 * Get the first item in the list of statuses
 * @since 1.6.0
 * @return String
 */
if( ! function_exists( 'ctdb_default_status' ) ) {
	function ctdb_default_status() {
		$statuses = ctdb_statuses_list();
		if( ! empty( $statuses ) ) {
			return $statuses[0];
		}
		return false;
	}
}

/**
 * Can the current user update the status?
 * @since 1.6.0
 * @return Boolean
 */
if( ! function_exists( 'ctdb_can_update_status' ) ) {
	function ctdb_can_update_status() {
		$pro_options = get_option( 'ctdb_pro_settings');
		if( isset( $pro_options['status_updaters'] ) ) {
			$updater = $pro_options['status_updaters'];
			switch( $updater ) {
				case 'op':
				// Only the Original Poster can update the status
				if( ctdb_user_is_original_poster() || ctdb_user_is_admin() ) {
					return true;
				}
				break;
				case 'contributor':
				// Any contributor to the topic can update the status
				if( ctdb_user_is_contributor() || ctdb_user_is_admin() ) {
					return true;
				}
				break;
			}
		}
		return false;
	}
}

/**
 * Is the current user the OP?
 * @since 1.6.0
 * @return Boolean
 */
if( ! function_exists( 'ctdb_user_is_original_poster' ) ) {
	function ctdb_user_is_original_poster() {
		$current_user = get_current_user_id();
		$author_id = get_the_author_meta( 'ID' );
		if( $current_user == $author_id ) {
			return true;
		}
		return false;
	}
}

/**
 * Is the current user the OP?
 * @since 1.6.0
 * @return Boolean
 */
if( ! function_exists( 'ctdb_user_is_admin' ) ) {
	function ctdb_user_is_admin() {
		if( current_user_can( 'manage_plugins' ) ) {
			return true;
		}
		return false;
	}
}

/**
 * Is the current user a contributor to this topic?
 * A contributor is either the OP or someone who has left a comment
 * @since 1.6.0
 * @return Boolean
 */
if( ! function_exists( 'ctdb_user_is_contributor' ) ) {
	function ctdb_user_is_contributor() {
		$current_user = wp_get_current_user();
		// If the user is the author
		if( ctdb_user_is_original_poster() ) {
			return true;
		} else {
			// See if the user has left a comment
			// Query the comments for the topic by user email
			$args = array(
				'post_id'		=> get_the_ID(),
				'author_email'	=> array( $current_user->user_email )
			);
			$comments = get_comments( $args );
			if( ! empty( $comments ) ) {
				// If the array is not empty, we must have found the current user's email address as a commenter
				return true;
			}
		}
		return false;
	}
}

/**
 * Return markup for status select
 * @since 1.6.0
 */
if( ! function_exists( 'ctdb_status_select' ) ) {
	function ctdb_status_select() {
		$select = '';
		// Check if we're using Status
		if( ctdb_is_status_enabled() ) {
			$status = ctdb_status();
			// Get the status fields
			$fields = ctdb_statuses_list();
			// Build the select
			if( ! empty( $fields ) && is_array( $fields ) ) {
				$select = wp_nonce_field( 'topic_status_nonce', 'topic_status_nonce' );
				$select .= '<select name="ctdb-status-select" id="ctdb-status-select">';
				foreach( $fields as $field ) {
					$field = trim( $field );
					$select .= '<option value="' . esc_attr( $field ) . '" ' . selected( $status, $field, false ) . '>' . esc_attr( $field ) . '</option>';
				}
				$select .= '</select>';
			}
		}	
		return $select;
	}
	
}

/**
 * Should be output the status or the status select?
 * @since 1.6.0
 * @return HTML
 */
if( ! function_exists( 'ctdb_status_output' ) ) {
	function ctdb_status_output() {
		// If we're allowed to update status, display the select
		if( ctdb_can_update_status() ) {
			$output = ctdb_status_select();
		} else {
			$output = ctdb_status();
		}
		return $output;
	}
}