<?php
/*
 * Discussion Board Categories class
*/

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Plugin public class
 **/
class CT_DB_Pro_Categories extends CT_DB_Public {
	
	public $show_icons = true;
	public $categories_enabled = true;
	public $tags_enabled = true;
	
	public function __construct() {
		//
	}
	
	/*
	 * Initialize the class and start calling our hooks and filters
	 * @since 1.1.0
	 */
	public function init() {
		
		add_action( 'init', array( $this, 'register_taxonomy' ) );
		add_action( 'init', array( $this, 'check_permissions' ) );
		
		add_filter( 'ctdb_topics_form_before_submit', array( $this, 'topic_form_filter' ) );
		add_filter( 'ctdb_post_args', array( $this, 'topic_form_submission_filter' ) );
		add_action( 'ctdb_new_topic_id', array( $this, 'create_new_term' ) );
		add_action( 'ctdb_new_topic_id', array( $this, 'add_tags' ) );
		
		add_filter( 'ctdb_filter_single_content_end', array( $this, 'display_categories_standard_layout' ), 30 );
		add_filter( 'ctdb_info_meta_close_classic_meta', array( $this, 'display_categories_classic_layout' ), 10 );
		add_filter( 'ctdb_meta_fields_end', array( $this, 'display_categories_table_layout' ), 30 );
		
	}
	
	/*
	 * Set some variables
	 * @since 1.2.0
	 */
	public function check_permissions() {
		$this->show_icons = ctdb_use_icons();
		// Categories enabled
		$options = get_option( 'ctdb_categories_settings' );
		if( ! isset( $options['allow_categories'] ) ) {
			$this->categories_enabled = false;
		}
		if( ! isset( $options['allow_tags'] ) ) {
			$this->tags_enabled = false;
		}
		
	}
	
	// Add categories if enabled
	public function topic_form_filter( $form ) {
		
		$options = get_option( 'ctdb_categories_settings' );
		
		// Add a select box for categories
		// @todo Display as checkboxes / radio buttons / multiselect ?
		if( ! empty( $options['allow_categories'] ) ) {
			$terms = get_terms ( 'topic-category', array ( 'hide_empty' => 0 ) );
			$term_select = '<select name="topic_category" id="topic_category" class="postform required ctdb_new_topic_select" tabindex="902">';
			$term_select .= '<option value="-1">' . __ ( 'Select a category', 'discussion-board-pro' ) . '</option>';
			// Add terms if they exist
			if ( ! empty ( $terms ) && ! is_wp_error ( $terms ) ) {
				foreach ( $terms as $term ) {
					$term_select .= '<option value="' . $term->term_id . '">' . $term->name . '</option>';
				}
			}
			// Add option to add new category
			if( ! empty( $options['create_categories'] ) ) {
				$term_select .= '<option value="new">'. __( 'Add new category', 'discussion-board-pro' ) . '</option>';
			}
			$term_select .= '</select>';
		}		
		
		// Add the select box to the $fields
		if( isset( $term_select ) && $term_select != '' ) {
			$form['fields'][] = $term_select;
		}
		
		// Add text input if user is allowed to add new category
		if( ! empty( $options['create_categories'] ) ) {
			$form['fields']['topic_category'] = '<input style="display: none;" type="text" name="new_topic_category" id="new_topic_category" placeholder="' . __( 'Enter new category name', 'discussion-board-pro' ) . '">';
		}
		
		// Add tags
		if( ! empty( $options['allow_tags'] ) ) {
			$form['fields']['topic_tags'] = '<input type="text" name="new_topic_tags" id="new_topic_tags" placeholder="' . __( 'Enter tags separated by commas', 'discussion-board-pro' ) . '">';
		}
		
		return $form;
		
	}
	
	/*
	 * Filter the submission form
	 * Add taxonomy term
	 * @since 1.1.0
	*/
	public function topic_form_submission_filter( $post_args ) {
		// Check for taxonomy
		if( isset( $_POST['topic_category'] ) ) {
			$terms = $_POST['topic_category'];
			$topic_category = array (
				'topic-category'	=> $terms
			);
			$post_args['tax_input'] = $topic_category;
		}
		return $post_args;
	}
	
	/*
	 * Create new taxonomy term
	 * @hooked to CT_DB_Public ctdb_new_topic_id
	 * @since 1.1.0
	*/
	public function create_new_term( $post_id ) {
		if( isset( $_POST['new_topic_category'] ) && ! empty( $_POST['new_topic_category'] ) ) {
			wp_insert_term( esc_attr( $_POST['new_topic_category'] ), 'topic-category' );
			wp_set_object_terms( $post_id, esc_attr( $_POST['new_topic_category'] ), 'topic-category' );
		}
	}
	
	/*
	 * Add tags
	 * @hooked to CT_DB_Public ctdb_new_topic_id
	 * @since 1.1.0
	*/
	public function add_tags( $post_id ) {
		if( isset( $_POST['new_topic_tags'] ) && ! empty( $_POST['new_topic_tags'] ) ) {
			$tags = explode( ',', $_POST['new_topic_tags'] );
			if( ! empty( $tags ) ) {
				foreach( $tags as $tag ) {
					wp_insert_term( esc_attr( $tag ), 'topic-tag' );
					wp_set_object_terms( $post_id, esc_attr( $tag ), 'topic-tag' );
				}
			}
		}
	}
	
	/*
	 * Display categories in standard layout on single.php
	 * @since 1.1.0
	*/
	public function display_categories_standard_layout( $content ) {
		/*
		 * Check the layout.
		 * Only display categories here if the layout is standard
		 * @since 1.2.0
		 */
		$design_options = get_option( 'ctdb_design_settings' );
		if( isset( $design_options['info_bar_layout'] ) && $design_options['info_bar_layout'] != 'standard' ) {
			return $content;
		}

		// Categories position
		$options = get_option( 'ctdb_categories_settings' );
		$position = $options['categories_position'];
		if( $position == 'hide' ) {
			return $content;
		}
		
		// Use icons?
		if( $this->show_icons ) {
			$heading_cat = '<span class="dashicons dashicons-category"></span>';
			$heading_tag = '<span class="dashicons dashicons-tag"></span>';
		} else {
			$heading_cat = __( 'Categorized: ', 'discussion-board-pro' );
			$heading_tag = __( 'Tagged: ', 'discussion-board-pro' );
		}
		$post_id = get_the_ID();
		
		$new_content = '';
		if( $this->categories_enabled ) {
			$new_content .= get_the_term_list( $post_id, 'topic-category', '<ul class="ctdb-categories"><li>' . $heading_cat, ',&nbsp;</li><li>', '</li></ul>' );
		}
		if( $this->tags_enabled ) {
			$new_content .= get_the_term_list( $post_id, 'topic-tag', '<ul class="ctdb-categories"><li>' . $heading_tag, ',&nbsp;</li><li>', '</li></ul>' );
		}
		
		if( $position == 'above-content' ) {
			return $new_content . $content;
		} else {
			return $content . $new_content;
		}
	}
	
	/*
	 * Display categories in table layout on single.php
	 * @since 1.2.0
	*/
	public function display_categories_table_layout( $output ) {
		/*
		 * Check the layout.
		 * Only display categories here if the layout is table
		 * @since 1.2.0
		 */
		$design_options = get_option( 'ctdb_design_settings' );
		if( ! isset( $design_options['info_bar_layout'] ) || $design_options['info_bar_layout'] != 'table' ) {
			return $output;
		}
		
		// Categories position
		$options = get_option( 'ctdb_categories_settings' );
		$position = $options['categories_position'];
		if( $position == 'hide' ) {
			return $output;
		}
		
		$post_id = get_the_ID();
		
		if( $this->categories_enabled ) {
			// Open wrapper for categories		
			$output .= '<div class="ctdb-info-title ctdb-info-cell">';
				if( $this->show_icons ) $output .= '<span class="dashicons dashicons-category"></span>';
				$output .= __( 'Categorized', 'discussion-board-pro' );
			$output .= '</div><!-- .ctdb-info-title -->';
			$output .= '<div class="ctdb-info-meta-wrap ctdb-info-cell">';
				$output .= '<div class="ctdb-info-meta">';
					$output .= get_the_term_list( $post_id, 'topic-category', '', ' ' );
				$output .= '</div><!-- .ctdb-info-meta -->';	
			$output .= '</div><!-- .ctdb-info-meta-wrap -->';
		}
		
		if( $this->tags_enabled ) {
			// Open wrapper for tags
			$output .= '<div class="ctdb-info-title ctdb-info-cell">';
				if( $this->show_icons ) $output .= '<span class="dashicons dashicons-tag"></span>';
				$output .= __( 'Tagged', 'discussion-board-pro' );
			$output .= '</div>';
			$output .= '<div class="ctdb-info-meta-wrap ctdb-info-cell">';
				$output .= '<div class="ctdb-info-meta ctdb-info-author">';
					$output .= get_the_term_list( $post_id, 'topic-tag', '', ' ' );
				$output .= '</div>';		
			$output .= '</div><!-- .ctdb-info-meta-wrap -->';
		}

		return $output;
		
	}
	
	/*
	 * Display categories in classic layout on single.php
	 * @since 1.4.0
	*/
	public function display_categories_classic_layout( $output ) {
		/*
		 * Check the layout.
		 * Only display categories here if the layout is classic
		 * @since 1.4.0
		 */
		$design_options = get_option( 'ctdb_design_settings' );
		if( ! isset( $design_options['info_bar_layout'] ) || $design_options['info_bar_layout'] != 'classic' ) {
			return $output;
		}
		
		// Categories position
		$options = get_option( 'ctdb_categories_settings' );
		$position = $options['categories_position'];
		if( $position == 'hide' ) {
			return $output;
		}
		
		$post_id = get_the_ID();
		
		$new_content = '';
		$category_content = '';
		
		if( $this->categories_enabled ) {
			// Open wrapper for categories
			$category_content .= __( 'Categorized: ', 'discussion-board-pro' );
			$category_content .= get_the_term_list( $post_id, 'topic-category', '', ' ' );
		}
		
		$tag_content = '';
		if( $this->tags_enabled ) {
			$tag_content .= __( 'Tagged: ', 'discussion-board-pro' );
			$tag_content .= get_the_term_list( $post_id, 'topic-tag', '', ' ' );
		}
		
		if( $category_content != '' || $tag_content != '' ) {
			// We've got either categories or tags to return
			$new_content = '<p>';
			if( $category_content != '' && $tag_content != '' ) {
				// We've got both types
				$new_content .= $category_content . '<br>' . $tag_content;
			} else {
				// We've only got one of these types
				$new_content .= $category_content;
				$new_content .= $tag_content;
			}
			
			$new_content .= '</p>';
		}
		
		$output .= $new_content;

		return $output;
		
	}
	
	
	/*
	 * Register the taxonomy
	 * @since 1.1.0
	 */
	public function register_taxonomy() {
		
		// Add topic category taxonomy
		// Only if option is selected
		$options = get_option ( 'ctdb_categories_settings' );
		if ( isset ( $options['allow_categories'] ) ) {
			$labels = array(
				'name'                       => _x( 'Topic Categories', 'taxonomy general name' ),
				'singular_name'              => _x( 'Topic Category', 'taxonomy singular name' ),
				'search_items'               => __( 'Search Topic Categories' ),
				'all_items'                  => __( 'All Topic Categories' ),
				'parent_item'                => __( 'Parent Topic Category' ),
				'parent_item_colon'          => __( 'Parent Topic Category:' ),
				'edit_item'                  => __( 'Edit Topic Category' ),
				'update_item'                => __( 'Update Topic Category' ),
				'add_new_item'               => __( 'Add New Topic Category' ),
				'new_item_name'              => __( 'New Topic Category Name' ),
				'menu_name'                  => __( 'Topic Categories' ),
			);
			$args = array(
				'hierarchical'          => true,
				'labels'                => $labels,
				'show_ui'               => true,
				'show_admin_column'     => true,
				'query_var'             => true,
				'rewrite'               => array( 'slug' => 'topic-category' ),
			);				
			register_taxonomy( 'topic-category', 'discussion-topics', $args );
		}
		
		if ( isset ( $options['allow_tags'] ) ) {
			$labels = array(
				'name'                       => _x( 'Topic Tags', 'taxonomy general name' ),
				'singular_name'              => _x( 'Topic Tag', 'taxonomy singular name' ),
				'search_items'               => __( 'Search Topic Tags' ),
				'all_items'                  => __( 'All Topic Tags' ),
				'parent_item'                => __( 'Parent Topic Tag' ),
				'parent_item_colon'          => __( 'Parent Topic Tag:' ),
				'edit_item'                  => __( 'Edit Topic Tag' ),
				'update_item'                => __( 'Update Topic Tag' ),
				'add_new_item'               => __( 'Add New Topic Tag' ),
				'new_item_name'              => __( 'New Topic Tag Name' ),
				'menu_name'                  => __( 'Topic Tags' ),
			);
			$args = array(
				'hierarchical'          => false,
				'labels'                => $labels,
				'show_ui'               => true,
				'show_admin_column'     => true,
				'query_var'             => true,
				'rewrite'               => array( 'slug' => 'topic-tag' ),
			);				
			register_taxonomy( 'topic-tag', 'discussion-topics', $args );
		}
		
	}
	
}